/********		exp3.C

In this example file, we analyze the behavior of
Chua's circuit whose only nonlinear element is a nonlinear
resistor whose i-v characterictic is given by
	i = m1*v + 0.5*(m0 - m1)(|v + 1| - |v - 1|)

	|------|------R---|-------|
	|      |          |       |
	L      C1         C2      NR
	|      |          |       |
	|------|----------|-------|

	This is a 3rd order autonomous system, and since the period
	of the system is not known ahead of time, we should choose
	a reasonable value for it. To make the stepsize h sufficiently
	small, we can for example take T=5, thus h=16*T/TNP is small
	enough. To start the system, the initial conditions are
	chosen different from zero.
	Capacitor voltages and the inductor current are taken as
	the state variables. For chaotic behavior, the nondimensional
	parameters are given as
	   a = 9.85, b = 100/7, m0 = -1/7 , m1 = 2/7.
	In our analysis, we'll keep m0,m1 constan and use a,b as the
	parameters to be manipulated.
********/


#include "head.h"
#include "memall.h"
#include "rk.h"
#include "gtools.h"
#include "four.h"


#define TNP 2048 /* Total # of points to be used for integration  */
#define NPS 1024 /* # of points to be stored for display	*/
#define NPV 2	/* # of variables to be plotted	*/
#define NV 3    /* # of state variables (order of system)	*/
#define NT 16	/* # of periods to be observed (t_end - t_begin = NT*T)  */
#define NPAR 2  /* # of parameters to be changed	*/

#define ULC 1 /* Upper left corner for display	*/
#define URC 2 /* Upper right corner for display	*/
#define LLC 3 /* Lower left corner for display	*/
#define LRC 4 /* Lower right corner for display	*/

char *parname="ab";
/* Declare the system parameters globally */
float par[]={9.85,100.0/7.0};
float dpar[]={0.05,0.05};

/* User-supplied 'derivative' function	*/

void derivs(float x, float *yt, float *dyt)
{
float gy1;
	gy1 = (2.0/7.0)*yt[1] - (3.0/14.0)*(fabs(yt[1]+1)-fabs(yt[1]-1));
		dyt[1] = par[0]*(yt[2] - gy1);
		dyt[2] = yt[1] - yt[2] + yt[3];
		dyt[3] = -1.0*par[1]*yt[2] + 0.0*x;
}


/*	Main Program */
void main(void)
{
float x1,T,h,t;
int i,c,j,xgrid,ygrid,k,c2,ind;
int flag=1,flag2=1,flag4=1,flag5 = 0,flag6 = 1;
float *vs,*yout,*dyt,*yend;
float *time,**A;
float *fft,*xfft,*yfft;
float ti,tf,yi,yf,xstart=0.0;
unsigned int size;
FILE *ss;
char fname[12];
void far *buff;

		/* Allocate Memory for the arrays	*/
		fft = vector(TNP+2);
		A = matrix(NPV,NPS);
		time = vector(NPS);
		xfft = vector(NPS/2);
		yfft = vector(NPS/2);
		yend = vector(NV+1);
	vs=vector(NV+1);
	dyt = vector(NV+1);
	yout = vector(NV+1);
					for(k=0;k<=NV;k++)
				     yend[k] = 0.20;
		/*	Initial values for the system parameters	*/

	 yi = -2.0;  /* Default values for  Ymin, Ymax for the Y-axis,	*/
	 yf = 2.0;   /* Xgrid and Ygrid  */
	 xgrid = 5;
	 ygrid = 4;
			grafini(); /* Initialize graphics mode	*/

			clearviewport();
		   setcolor(12);
		 gprintfxy(100,240,"Running... Please Wait...");
		 settextstyle(SMALL_FONT,HORIZ_DIR,0);

				do{
					if (flag){
			T = 5.0;
		h = T*(float) NT/TNP;

			if (flag6){
			  xstart = 0.0;
			     for(k=0;k<=NV;k++)
			  yend[k] = 0.20;
				}
	     for(k=0;k<=NV;k++)
		vs[k]=  yend[k];/*Initial conditions	*/
			x1 =  xstart; /* Initial time	*/


/* Allow NT*T time to pass to eliminate transients	*/
		if (flag6)
	for(i=1;i<=TNP;i++){
			derivs(x1,vs,dyt);
	rk4(vs,dyt,NV,x1,h,yout,derivs);
	     for(k=0;k<=NV;k++)
		vs[k]=  yout[k];
			x1 += h;
		}

			t = x1;
			       /*Solve and store for display	*/
	for(i=0;i<TNP;i++){
			derivs(x1,vs,dyt);
	rk4(vs,dyt,NV,x1,h,yout,derivs);
			if (!(i%2)){
			    j = i/2;
	     for(k=0;k<NPV;k++)
		A[k][j] =  yout[k+1];
			time[j] = t + i*h;
				}
		     for(k=0;k<=NV;k++)
		  vs[k]= yout[k];
			x1 += h;
		}
			/* Control to set Xmin,Xmax to defaults	*/
		if (flag4)
	 ti = t, tf = x1;

			if (flag5)
			   ti = t,tf = x1,flag5 = 0;

			}




/* Call Plotting utilities	*/

			 if (flag2){

			setviewport(0,0,639,479,1);
			clearviewport();


	plotxy(time,A[0],NPS-1,ti,tf,yi,yf,14,xgrid,ygrid,ULC);

	plotxy(time,A[1],NPS-1,ti,tf,yi,yf,14,xgrid,ygrid,URC);

	plotxy(A[0],A[1],NPS-1,yi,yf,yi,yf,14,ygrid,ygrid,LLC);
		setviewport(0,0,639,479,1);
			for(k=0;k<NPAR;k++)
		  gprintfxy(590,270+k*15,"%c=%.2f",parname[k],par[k]);
				}


/* Use key code mapping to be able to use Alt- and Ctrl- keys  */

		c = bioskey(0); /* Get user request	*/
/*Change system parameters	*/
			  c2 = (c & 0xFF);
			   if (isalpha(c2)){
			     ind = index(parname,c2);
				if (ind != -1){
  par[ind] = (isupper(c2))?par[ind]+dpar[ind]:par[ind]-dpar[ind];
	flag = flag2 = flag6 = 1;
			}
			else
			  flag2 = flag = 0;
				}
				else
			switch(c){



				/* Alt-i  */

	case 0x1700  : /*Increase the time interval by NT*T	*/
			flag6 = 0;
			  xstart = t = x1;
			    for(k=0;k<=NV;k++)
			  yend[k] = vs[k];
	setviewport(320,465,639,479,1);
	clearviewport();
	setcolor(12);
	  gprintf("Running.. Wait..");
	for(i=0;i<TNP;i++){
			derivs(x1,vs,dyt);
	rk4(vs,dyt,NV,x1,h,yout,derivs);
			if (!(i%2)){
				j = i/2;
			      for(k=0;k<NPV;k++)
				A[k][j] = yout[k+1];
		time[j] = t + i*h;
				}
				   for(k=0;k<=NV;k++)
				  vs[k] = yout[k];
			x1 += h;
		}
		ti = t;
		tf = x1;

		  flag = 0;
		  flag2 = 1;
		  flag4 = 1;
		  flag5 = 1;
		  break;

				/* Alt-x */
	case 0x2d00 : /*Change Xmin, Xmax	*/

		setviewport(320,465,639,479,1);
		clearviewport();
  gprintf("Enter Xmin and Xmax:");
		      gscanf("%f %f",&ti,&tf);
		 flag = 0;
		 flag2 = 1;
		 flag4 = 0;
		   if (ti == 999.0 && tf == 999.0){
			ti = t;
			tf = x1;
			   flag4 = 1;
			     }



			break;

				/* Alt-y */
	case 0x1500 :	/* Change Ymin,Ymax	*/

			setviewport(320,465,639,479,1);
			clearviewport();
		  gprintf("Enter Ymin and Ymax:");
		      gscanf("%f %f",&yi,&yf);
			flag = 0;
			flag2 = 1;
			break;

			/* Alt-g  */
			/* To change Xgrid,Ygrid  */
	case 0x2200 :
			setviewport(320,465,639,479,1);
			clearviewport();
		  gprintf("Enter Xgrid and Ygrid:");
		      gscanf("%d %d",&xgrid,&ygrid);
			flag = 0;
			flag2 = 1;
			break;

	case 0x1f00 :	/* Alt-s  to save screen to a file  */

			setviewport(320,465,639,479,1);
			clearviewport();
		  gprintf("Screen Dump File Name(Ext .IMG):");
		      gscanf("%s",fname);
		      if ((ss = gopen(fname,"wb",".img")) != NULL){
				clearviewport();
		       setviewport(0,0,639,479,1);
				sscreen(ss);
			setviewport(320,465,639,479,1);
			clearviewport();
	gprintfxy(5,0,"Screen image saved to '%s'",fname);
			      fclose(ss);
				}
				else{
				  clearviewport();
	gprintfxy(5,0,"Invalid filename -- Screen not saved");
				}
			flag = 0;
			flag2 = 0;
			break;

	case 0x1300 : /* Alt-r, to restore a previously saved screen */
			setviewport(320,465,639,479,1);
			clearviewport();
		  gprintf("File Name to be Restored(Ext .IMG):");
		      gscanf("%s",fname);
		      if ((ss = gopen(fname,"rb",".img")) != NULL){
		       setviewport(0,0,639,479,1);
				clearviewport();
				rscreen(ss);
			setviewport(320,465,639,479,1);
			clearviewport();
	gprintfxy(5,0,"Screen image restored from '%s'",fname);
			      fclose(ss);
				}
				else{
				  clearviewport();
	gprintfxy(5,0,"Invalid filename -- Screen not restored");
					}

			flag = 0;
			flag2 = 0;
			break;

	case 0x1400 : /* Alt-t,  Do FFT  */
			setviewport(320,465,639,479,1);
			setcolor(12);
			gprintfxy(10,0,"FFT... Wait...");
		  for(j=0;j<NPS;j++){
		    fft[2*j+1] = A[0][j];
		    fft[2*j+2] = 0.0;
				}
		       fourout(fft,xfft,yfft,NPS,2*h);
			  clearviewport();
	 plotxy(xfft,yfft,NPS/2-1,0,5,-50,0,14,5,5,LRC);

		flag = 0;
		flag2 = 0;
			break;


	 case 0x2300 :	/* Help	*/
			setviewport(350,250,550,450,1);
			size = imagesize(0,0,200,200);
			buff = farmalloc(size*sizeof(char));
			getimage(0,0,200,200,buff);
			clearviewport();
				setcolor(12);
			    for(i=0;i<9;i++)
	    gprintfxy(0,15*i,"%s",help[i]);
		gprintfxy(10,160,"%s",help[9]);
			getch();
			clearviewport();
			putimage(0,0,buff,COPY_PUT);
				farfree(buff);
				setviewport(320,465,639,479,1);
			flag = 0;
			flag2 = 0;
			  break;



	default:        flag = 0;
			flag2 = 0;
			break;
		}
			if (flag){
		setviewport(320,465,639,479,1);
		clearviewport();
		   setcolor(12);
		 gprintf("Running.. Wait...");

					   }
					   /* ESC to exit */
			}while( c != 0x011b); /* Job Done */

		closegraph();   /* End Graphics mode	*/

		/* Free allocated memory	*/
		free_vector(fft);
	free_matrix(A,NPV);
	free_vector(time);
	free_vector(xfft);
	free_vector(yfft);
	free_vector(vs);
	free_vector(dyt);
	free_vector(yout);
	free_vector(yend);
		return;	/* End Program	*/
}





































