
/*****	GTOOLS.H
 --  A set of text input and output routines in graphics
mode, and X-Y plotting routines.
*********/

/*  A graphics based printf function. It will print the output
 *  at location xloc, yloc. It does not effect the current position.
 *  Assumes LEFT_TEXT and HORIZ_DIR justification.
 */
	/* Help Screen */
char *help[]={"Alt-x: Change Xmin,Xmax","Alt-y: Change Ymin,Ymax",\
"Alt-i: Increase Observation Time","Alt-g: Change Xgrid,Ygrid",\
"Alt-t: FFT","Alt-s: Save Screen to File","Alt-r: Restore Screen from File",\
"Alt-h: This Help Screen","ESC  : Quit","Any Key to Resume"};



int gprintfxy(int xloc, int yloc, char *fmt, ... )
{
    va_list argptr;                  /* Pointer to argument list */
    char str[BUFSIZE];               /* Buffer to build string into */
    int cnt;                         /* Result of sprintf conversion */
    struct fillsettingstype oldfill; /* Current fill setting */
    char userfillpattern[8];         /* Current user-fill pattern */

    va_start(argptr,fmt);            /* Initialize va_ functions */
    cnt = vsprintf(str,fmt,argptr);  /* Prints string to buffer */
    if (str[0] == NULL) return(0);

    /* Clear the space where the text is to be printed */
    getfillsettings(&oldfill);
    if (oldfill.pattern == USER_FILL)
	getfillpattern(userfillpattern);
    setfillstyle(SOLID_FILL,getbkcolor());
    bar(xloc,yloc,xloc + textwidth(str),yloc+textheight("H")*5/4);
    if (oldfill.pattern == USER_FILL)
	setfillpattern(userfillpattern,oldfill.color);
    else
	setfillstyle(oldfill.pattern,oldfill.color);
    outtextxy(xloc,yloc,str);        /* Write string to screen */
    va_end(argptr);                  /* Terminate va_ functions */
    return(cnt);                     /* Return the conversion count */
}

/*  A graphics based printf function. It will update the current
 *  position. Assumes LEFT_TEXT and HORIZ_DIR justification.
 */
int gprintf(char *fmt, ... )
{
    va_list argptr;                    /* Pointer to argument list */
    char str[BUFSIZE];                 /* Buffer to build string into */
    int cnt;                           /* Result of sprintf conversion */
    struct fillsettingstype oldfill;   /* Current fill setting */
    char userfillpattern[8];           /* Current user-fill pattern */
    int xloc, yloc;                    /* Current current position */

    va_start(argptr,fmt);              /* Initialize va_ functions */
    cnt = vsprintf(str,fmt,argptr);    /* Prints string to buffer */
    if (str[0] == NULL) return(0);
    /* Clear the space where the text is to be printed */
    xloc = getx();  yloc = gety();
    getfillsettings(&oldfill);
    if (oldfill.pattern == USER_FILL)
        getfillpattern(userfillpattern);
    setfillstyle(SOLID_FILL,getbkcolor());
    bar(xloc,yloc,xloc+textwidth(str),yloc+textheight("H")*5/4);
    if (oldfill.pattern == USER_FILL)
	setfillpattern(userfillpattern,oldfill.color);
    else
        setfillstyle(oldfill.pattern,oldfill.color);
    outtext(str);                    /* Write string to screen */
    va_end(argptr);                  /* Terminate va_ functions */
    return(cnt);                     /* Return the conversion count */
}



/*  A graphics based text input routine. Returns the string entered.
 *  Echos text as it is entered. It supports the backspace character.
 */
char *ggets(char *buffer)
{
    int currloc, maxchars, oldcolor;
    struct viewporttype view;
    char ch, charbuff[3];

    buffer[0] = '\0';
    currloc = 0;
    getviewsettings(&view);
    maxchars = (view.right - getx()) / textwidth("M") - 1;
    if (maxchars <= 0) return(NULL);
    gprintfxy(getx(), gety(), "_");
    while ((ch=getch()) != CR) {
	if (ch == BS) {
	    if (currloc > 0) {
		currloc--;
		if (currloc <= maxchars) {
		    oldcolor = getcolor();
		    setcolor(getbkcolor());
		    sprintf(charbuff,"%c",buffer[currloc]);
		    gprintfxy(getx()-textwidth(charbuff),gety(),"%c_",
						 buffer[currloc]);
		    setcolor(oldcolor);
		    moveto(getx()-textwidth(charbuff),gety());
                }
            }
        }
        else {
            if (currloc < maxchars) {
                oldcolor = getcolor();
                setcolor(getbkcolor());
                gprintfxy(getx(),gety(),"_");
                setcolor(oldcolor);
		buffer[currloc] = ch;
		gprintf("%c",ch);
		currloc++;
	    }
	    else
		putch(0x07);
	}
	if (currloc < maxchars)
	    gprintfxy(getx(),gety(),"_");
    }
    if (currloc <= maxchars) {
	oldcolor = getcolor();
	setcolor(getbkcolor());
	gprintfxy(getx(),gety(),"_");
	setcolor(oldcolor);
    }
    buffer[currloc] = '\0';
    return(buffer);
}

/*  A graphics based scanf() function. It updates the current position.
 *  Assumes LEFT_TEXT and HORIZ_DIR justification.
 */
int gscanf(char *fmt, ... )
{
    va_list argptr;                  /* Argument list pointer */
    char str[BUFSIZE];               /* Buffer to accept user input */
    int cnt;                         /* Number of fields converted */

    va_start(argptr,fmt);            /* Initialize va_ functions */
    ggets(str);                      /* Get the user input */
    cnt = vsscanf(str,fmt,argptr);   /* Convert the input according */
				     /* to the format string */
    va_end(argptr);                  /* Terminate the va_ functions */
    return(cnt);                     /* Return the # of successful */
				     /* input conversions */
}

/*  A graphics based scanf(). It does not effect the current
 *  position and will echo input as it is entered at (xloc, yloc).
 *  Assumes LEFT_TEXT and HORIZ_DIR justification.
 */
int gscanfxy(int xloc, int yloc, char *fmt, ... )
{
    va_list argptr;                 /* Pointer to argument list */
    char str[BUFSIZE];              /* Buffer to accept user input */
    int cnt;                        /* Number of conversions made */
    int oldx, oldy;                 /* Original current position */

    oldx = xloc;                    /* Save current position so that */
    oldy = yloc;                    /* it can later be restored */
    moveto(xloc,yloc);              /* Move to the new current position */
    va_start(argptr,fmt);           /* Initialize the va_ functions */
    ggets(str);                     /* Get the user's input */
    cnt = vsscanf(str,fmt,argptr);  /* Convert input string */
    va_end(argptr);                 /* Terminate va_ function */
    moveto(oldx,oldy);              /* Restore current position */
    return(cnt);                    /* Return conversion count */
}

/*****
 A general graphics routines with full and
** partial screen plotting capabilities,
screen saving and restoring (to-from files)
*******/


int xoff;
int yoff;


int hsw; /* Horizontal size for each of four pairs of X-Y axes */
int vsw; /* Vertical size for each of four pairs of X-Y axes */
/* These constants are used with "putframe.. putnumbers.. and plotxy..
functions.  */

void grafini(void)
{
int gdriver,gmode,errcode;
	detectgraph(&gdriver,&gmode);
	initgraph(&gdriver,&gmode,"");
	errcode = graphresult();
		if (errcode < 0){
		 printf("%s\n",grapherrormsg(errcode));
		 exit(1);
			}
}



void putframew(int xgrid, int ygrid)
{
/* Draws the frame of X-Y axis, which is hsw pixels
horizontal and vsw pixels vertical. User specifies the number of
grids for X-Y axes. Each grid has 5 divisions. Each coordinate
system can have n number of plots. Upper left corner of X-Y
is calculated by (xoff,yoff). Can have a maximum of 4
nonoverlapping coordinate system of same size.
*/

int i,j;
int xg,yg;
	xg=hsw/xgrid;
	yg=vsw/ygrid;
	  setbkcolor(8);
	  setcolor(3);
			setlinestyle(SOLID_LINE,0xFFFF,THICK_WIDTH);
			line(xoff,yoff,xoff+hsw,yoff);
			line(xoff+hsw,yoff,xoff+hsw,yoff+vsw);
			line(xoff+hsw,yoff+vsw,xoff,yoff+vsw);
			line(xoff,yoff+vsw,xoff,yoff);
	for (i=0;i<ygrid;i++){
	  line(xoff-5,i*yg+yoff,xoff+5,i*yg+yoff);
			setlinestyle(DOTTED_LINE,0xFFFF,NORM_WIDTH);
		  line(xoff,i*yg+yoff,xoff+hsw,i*yg+yoff);
	for (j=1;j<=5;j++)
    line(xoff-3,j*yg/5+yoff+i*yg,xoff+3,j*yg/5+yoff+i*yg);
			setlinestyle(SOLID_LINE,0xFFFF,THICK_WIDTH);
			}
	  for (i=0;i<xgrid;i++){
	    line(xoff+i*xg,yoff+vsw-5,xoff+i*xg,yoff+vsw+5);
			setlinestyle(DOTTED_LINE,0xFFFF,NORM_WIDTH);
		    line(xoff+i*xg,yoff+vsw,xoff+i*xg,yoff);
	for (j=1;j<=5;j++)
	  line(xoff+j*xg/5+i*xg,yoff+vsw-3,j*xg/5+xoff+i*xg,yoff+vsw+3);
			}
}





void putwinw(float xmin,float xmax,float ymin,float ymax,int xgrid,\
int ygrid,int region)
{
/* Puts the actual numbers for X-Y axes at the grid level. Number
of grids is given by xgrid and ygrid. Location of the numbers
is given by (xm+40-hsw+i*xg,20+ym) for X-axis and (xm-hsw,ym+10-vsw+j*yg)
for Y-axis where xg=hsw/xgrid, yg=vsw/ygrid and i:0--->xgrid
and j:ygrid--->0. Works with function 'putframew'.
*/

int i,xg,yg;
float dx,dy;
char buf[7];
	setviewport(0,0,639,479,1);
			if (region==0){
				xoff=40;
				yoff=10;
				hsw=500;
				vsw=420;
					}
					 else{
					    hsw=240;
					    vsw=200;
						}
			if (region==1){
				xoff=40;
				yoff=5;
					}
			if (region==2){
				xoff=340;
				yoff=5;
					}
			if (region==3){
				xoff=40;
				yoff=240;
					}
			if (region==4){
				xoff=340;
				yoff=240;
					}


		yg=vsw/ygrid;
		xg=hsw/xgrid;
		   putframew(xgrid,ygrid);
	setlinestyle(SOLID_LINE,0XFFFF,NORM_WIDTH);
	dx= (xmax-xmin)/xgrid;
	dy=(ymax-ymin)/ygrid;
	    for(i=0;i<=xgrid;i++){
	    sprintf(buf,"% .2f",xmin + i*dx);
 outtextxy(xoff+i*xg-textwidth(buf)/2,yoff+vsw+10,buf);
		}
	    for(i=ygrid;i>=0;i--){
	    sprintf(buf,"% .2f",ymax - i*dy);
 outtextxy(xoff-40,yoff+i*yg-textheight(buf)/2,buf);
	}
		     setviewport(xoff,yoff,xoff+hsw,yoff+vsw,1);

}






void plotxy(float *x,float *y,int np,float xmin,float xmax,\
float ymin,float ymax,int color,int xg,int yg,int region)
{
/* Draw the plot of Y(array) versus Y(array) with np number of points
and in the color specified by color. Upper left corner of X-Y is
given by (xm+40-hsw,ym+10-vsw). Window size is hsw by vsw pixels.
Used in connection with "putframew and putnumbersw" functions.
*/

float xsc,ysc,xr,yr,xsc0,ysc0;
int i;
		xr=xmax - xmin;
		yr=ymax - ymin;

		putwinw(xmin,xmax,ymin,ymax,xg,yg,region);
	xsc0=  hsw*(x[0]-xmin)/xr;
	ysc0=  vsw*(ymax-y[0])/yr;
	    moveto((int) xsc0,(int) ysc0);
			setcolor(color);
		for(i=1;i<=np;i++){
	xsc=  hsw*(x[i]-xmin)/xr;
	ysc=  vsw*(ymax-y[i])/yr;
		lineto((int) xsc,(int) ysc);
			}
}



FILE *gopen(char *s, char *mode, char *ext)
{
FILE *fp;
		strcat(s,ext);
	fp=fopen(s,mode);
		  return (fp);
}



void sscreen(FILE *fp)
{
int i,t1=0,b=470;
int la1[]={0,210,420};
int ra1[]={210,420,630};
void *buf;
unsigned int imsize;
				for(i=0;i<3;i++){
			imsize=imagesize(la1[i],t1,ra1[i],b);
				buf= (void *) malloc(imsize);
	if (!buf)
	errmsg("Save screen MAF");
				getimage(la1[i],t1,ra1[i],b,buf);
				fwrite(buf,imsize,1,fp);
				free(buf);
					}


}


void rscreen(FILE *fp)
{
int fh,i;
int la1[]={0,210,420};
void *buf;
unsigned long fsize,fss;
			fh=fileno(fp);
			fsize=filelength(fh);
			fss=fsize/3;
			rewind(fp);
				for(i=0;i<3;i++){
				  buf= (void *) malloc(fss);
				  fread(buf,(unsigned) fss,1,fp);
				  putimage(la1[i],0,buf,COPY_PUT);
				  free(buf);
					}
}



















































